
/* Test driver for text deltas */

#include <ctype.h>
#include <stdio.h>
#include <apr_general.h>
#include "svn_delta.h"
#include "svn_error.h"

static svn_error_t*
read_from_file (void* data, char* buffer, apr_off_t* len)
{
  FILE* stream = data;
  if (feof (stream) || ferror (stream))
    *len = 0;
  else
    *len = fread (buffer, 1, *len, stream);
  return NULL;
}


static void
print_delta_window (svn_delta_window_t* window, FILE* stream)
{
  int i;
  apr_off_t off;

  fputs ("(WINDOW", stream);
  for (i = 0; i < window->num_ops; ++i)
    {
      apr_off_t const offset =  window->ops[i].offset;
      apr_off_t const length =  window->ops[i].length;
      switch (window->ops[i].op)
        {
        case svn_delta_source:
          fprintf (stream, "\n  (SOURCE %ld %ld)",
                   (long) offset, (long) length);
          break;
        case svn_delta_target:
          fprintf (stream, "\n  (TARGET %ld %ld)",
                   (long) offset, (long) length);
          break;
        case svn_delta_new:
          fputs ("\n  (INSERT \"", stream);
          for (off = offset; off < offset + length; ++off)
            {
              int const dat = window->new->data[off];
              if (iscntrl (dat) || !isascii(dat))
                fprintf (stream, "\\%3.3o", dat);
              else if (dat == '\\')
                fputs ("\\\\", stream);
              else
                putc (dat, stream);
            }
          fputs ("\")", stream);
          break;
        default:
          fputs ("\n  (BAD-OP)", stream);
        }
    }
  fputs (")\n", stream);
}


int
main (int argc, char **argv)
{
  FILE *source_file = NULL;
  FILE *target_file = NULL;
  svn_delta_stream_t* stream;
  svn_delta_window_t* window;

  if (argc == 2)
    {
      target_file = fopen (argv[1], "rb");
    }
  else if (argc == 3)
    {
      source_file = fopen (argv[1], "rb");
      target_file = fopen (argv[2], "rb");
    }
  else
    {
      fprintf (stderr, "Need two or three arguments.\n");
      exit (1);
    }

  apr_initialize();
  svn_text_delta ((source_file ? read_from_file : NULL),
                  source_file,
                  read_from_file,
                  target_file,
                  &stream);

  do {
    svn_next_delta_window (stream, &window);
    if (window != NULL)
      {
        print_delta_window (window, stdout);
        svn_free_delta_window (window);
      }
  } while (window != NULL);

  svn_free_delta_stream (stream);
  if (source_file)
    fclose (source_file);
  fclose (target_file);

  apr_terminate();
  exit (0);
}



/* 
 * local variables:
 * eval: (load-file "../../svn-dev.el")
 * end:
 */
